from datetime import date, datetime, timedelta
from typing import Optional

import pytest
import pytz

from psi_collector.collector_runner import CollectorRunner
from psi_collector.custom_collector import CustomCollector
from test_utils.mock import MockApi

TEST_CAPTURE_TIME = datetime.fromisoformat("2021-06-01T00:00:00").replace(tzinfo=pytz.utc)
EXPECTED_DATA = [{"dummy": "data"}, {"empty": ""}, {"number": 0}]


class TestCollector(CustomCollector):
    def __init__(self, capture_time, api, aws_session, http_session):
        super().__init__("test", capture_time, api, aws_session, http_session)

    def _collect(self, s3_path: Optional[str] = None):
        return EXPECTED_DATA

    def start_date(self, capture_time):
        return capture_time

    def end_date(self, capture_time):
        return capture_time


TestCollector.__test__ = False


class TestCollectorRunner_UnitTests:
    @pytest.mark.unit
    def test_collect(self):
        # Set up mocks
        mock_s3_loader = MockApi()
        mock_s3_loader.on("load", "test", EXPECTED_DATA, TEST_CAPTURE_TIME).return_val(None)
        mock_timestream_loader = MockApi()
        mock_timestream_loader.on(
            "load",
            data=EXPECTED_DATA,
            capture_time=TEST_CAPTURE_TIME,
            source="test",
            upsert=True,
            ignore_parse_error=True,
            max_batch_size=None,
            min_common_dimensions=None,
            is_multi=False,
        ).return_val({"invalid_items": [], "rejected_records": []})

        # Test collect
        runner = CollectorRunner(
            collectors={"test": TestCollector},
            s3_loader=mock_s3_loader,
            timestream_loader=mock_timestream_loader,
        )
        runner.collect({"collector_name": "test", "capture_time": TEST_CAPTURE_TIME.isoformat()})
        mock_s3_loader.assert_expectations()
        mock_timestream_loader.assert_expectations()

    @pytest.mark.unit
    def test_collect_invalid_items(self):
        # Set up mocks
        mock_s3_loader = MockApi()
        mock_s3_loader.on("load", "test", EXPECTED_DATA, TEST_CAPTURE_TIME).return_val(None)
        mock_timestream_loader = MockApi()
        mock_timestream_loader.on(
            "load",
            data=EXPECTED_DATA,
            capture_time=TEST_CAPTURE_TIME,
            source="test",
            upsert=True,
            ignore_parse_error=True,
            max_batch_size=None,
            min_common_dimensions=None,
            is_multi=False,
        ).return_val({"invalid_items": [1], "rejected_records": []})

        # Test collect with invalid items
        runner = CollectorRunner(
            collectors={"test": TestCollector},
            s3_loader=mock_s3_loader,
            timestream_loader=mock_timestream_loader,
        )

        with pytest.raises(Exception) as e_info:
            runner.collect(
                {"collector_name": "test", "capture_time": TEST_CAPTURE_TIME.isoformat()}
            )
        assert str(e_info.value).startswith("Failed to write to Timestream with invalid items")
        mock_s3_loader.assert_expectations()
        mock_timestream_loader.assert_expectations()

    @pytest.mark.unit
    def test_collect_rejected_records(self):
        # Set up mocks
        mock_s3_loader = MockApi()
        mock_s3_loader.on("load", "test", EXPECTED_DATA, TEST_CAPTURE_TIME).return_val(None)
        mock_timestream_loader = MockApi()
        mock_timestream_loader.on(
            "load",
            data=EXPECTED_DATA,
            capture_time=TEST_CAPTURE_TIME,
            source="test",
            upsert=True,
            ignore_parse_error=True,
            max_batch_size=None,
            min_common_dimensions=None,
            is_multi=False,
        ).return_val({"invalid_items": [], "rejected_records": [1]})

        # Test collect with rejected records
        runner = CollectorRunner(
            collectors={"test": TestCollector},
            s3_loader=mock_s3_loader,
            timestream_loader=mock_timestream_loader,
        )

        with pytest.raises(Exception) as e_info:
            runner.collect(
                {"collector_name": "test", "capture_time": TEST_CAPTURE_TIME.isoformat()}
            )
        assert str(e_info.value).startswith("Failed to write to Timestream with rejected records")
        mock_s3_loader.assert_expectations()
        mock_timestream_loader.assert_expectations()

    @pytest.mark.unit
    def test_get_capture_time(self):
        event = {
            "capture_time": "2021-07-31T00:00:00",
            "s3_path": "s3://bucket/path/to/object/1627689600000",
        }
        expected_capture_time = datetime.fromisoformat("2021-07-31T00:00:00").replace(
            tzinfo=pytz.utc
        )
        runner = CollectorRunner({"test": TestCollector}, MockApi(), MockApi())

        # Test get_capture_time from capture_time
        dt = runner.get_capture_time(event)
        assert dt == expected_capture_time

        # Test get_capture_time from s3_path
        event.pop("capture_time")
        dt = runner.get_capture_time(event)
        assert dt == expected_capture_time

        # Test get_capture_time without capture_time or s3_path
        event.pop("s3_path")
        dt = runner.get_capture_time(event)
        assert dt.tzinfo == pytz.utc

    @pytest.mark.unit
    def test_get_collector(self):
        runner = CollectorRunner({"test": TestCollector}, MockApi(), MockApi())
        expected_capture_time = datetime.now().astimezone(pytz.utc)
        # Test exception raised with a bad collector name
        with pytest.raises(Exception) as e_info:
            runner.get_collector("bad collector name", capture_time=expected_capture_time)
        assert str(e_info.value).startswith("Unrecognized collector")

        collector = runner.get_collector(
            "test", aws_session=MockApi(), capture_time=expected_capture_time
        )
        assert type(collector) == TestCollector

    @pytest.mark.unit
    def test_parse_ago_time(self):

        # Test collect
        runner = CollectorRunner(
            collectors={"test": TestCollector}, s3_loader=MockApi(), timestream_loader=MockApi()
        )
        capture_time = runner.get_capture_time({"capture_time": "42069ms"})
        assert capture_time.date() == (datetime.now() - timedelta(milliseconds=42069)).date()



import logging
from datetime import datetime
from typing import Any, Optional

from typeguard import typechecked

from psi_elt.abstract_runner import AbstractRunner, NoneType
from psi_elt.custom_loader import CustomLoader
from psi_elt.utils import filter_dict


@typechecked
class LoaderRunner(AbstractRunner):
    def __init__(
        self,
        loaders: dict[str, Any],
        aws_wrangler: Optional[Any] = None,
        logger: Optional[logging.Logger] = None,
    ):
        """
        Initializes a LoaderRunner.

        Accepts a loaders dict, mapping loader name to loader class.
        For example:
        {
            "slater": SlaterLoader,
            "tep": TepLoader
        }
        """
        super().__init__(jobs=loaders, logger=logger)
        self.__aws_wrangler = aws_wrangler

    def parse_event(self, event: dict[str, Any]) -> dict[str, Any]:
        """Parses a loader event. Child classes may override this function to
        implement custom event parsing rules.

        #### Required event keys
           1. name : the name of the loader to run

        #### Optional event keys
           1. capture_time                   : target time for the job to run against
           2. is_test                        : boolean for whethor or not this is a test run
           3. input_options.s3_uri           : S3 location to read data from
           4. input_options.s3_path          : S3 path to read data from
           5. output_options.s3_uri          : S3 location to write data to
           6. output_options.s3_path         : S3 path to write data to
           7. output_options.write_to_athena : boolean for whether or not to write to Athena
           8. output_options.athena_database : Athena database to write data to
           9. output_options.athena_table    : Athena table to write data to
        """
        parsed_event = {
            "name": event.get("name"),
            "capture_time": self.get_capture_time(event),
            "job_init_kwargs": {},
            "is_test": bool(event.get("is_test")),
            "load_kwargs": {"input_options": {}, "output_options": {}},
        }

        # Parse input options
        parsed_event["load_kwargs"]["input_options"] = filter_dict(
            dct=event.get("input_options"), keys=["s3_uri", "s3_path"]
        )

        # Parse output options
        parsed_event["load_kwargs"]["output_options"] = filter_dict(
            dct=event.get("output_options"),
            keys=["s3_uri", "s3_path", "write_to_athena", "athena_database", "athena_table"],
        )
        return parsed_event

    def validate_parsed_event(self, parsed_event: dict[str, Any]):
        """Validates that a parsed event has the required format"""
        schema = {
            "name": str,
            "capture_time": datetime,
            "job_init_kwargs": {},
            "is_test": bool,
            "load_kwargs": {
                "input_options": {"s3_uri": [str, NoneType], "s3_path": [str, NoneType]},
                "output_options": {
                    "s3_uri": [str, NoneType],
                    "s3_path": [str, NoneType],
                    "write_to_athena": [bool, NoneType],
                    "athena_database": [str, NoneType],
                    "athena_table": [str, NoneType],
                },
            },
        }
        try:
            self._validate_dict(parsed_event, schema)
        except AssertionError as e:
            self.logger.error("Invalid event")
            raise e

    def _run(
        self,
        name: str,
        capture_time: datetime,
        job_init_kwargs: dict,
        is_test: bool,
        load_kwargs: dict,
    ) -> CustomLoader:
        job_init_kwargs["aws_wrangler"] = self.__aws_wrangler
        loader: CustomLoader = self.get_job(name, capture_time, job_init_kwargs)
        loader.load(is_test, load_kwargs)
        return loader



from typeguard import typechecked
from typing import Any


@typechecked
def filter_dict(dct: Any, keys: list, accept_none: bool = False) -> dict:
    """Returns a dict filtered to the given keys (if they exist) and optionally ignores None"""
    if isinstance(dct, dict):
        return {k: v for k, v in dct.items() if k in keys and (accept_none or v is not None)}
    return {}



# coding: utf-8

"""
PSI_Collector

Module for testing.   # noqa: E501
"""

from setuptools import setup, find_packages  # noqa: H301

NAME = "psi_collector"
VERSION = "0.7.1"
REQUIRES = ["boto3", "pytz", "requests", "typeguard"]

# To install the library, run the following
#
# python setup.py install 
#
# prerequisite: setuptools
# http://pypi.python.org/pypi/setuptools

setup(
    name=NAME,
    version=VERSION,
    description="PSI Collector module",
    author="Pearson Service Insights",
    author_email="TOIOServiceWorkflowAutomation@grp.pearson.com",
    url="https://github.com/orgs/pearson-digitalandtechnology/POE-python-packages",
    keywords=["Pearson", "PSI", "Collector", "PSI_Collector"],
    install_requires=REQUIRES,
    packages=find_packages(),
    include_package_data=True,
    long_description="""\
     Module containing collectors that extract from apis.   # noqa: E501
    """,
)
